import 'dart:async';

import 'package:flutter/foundation.dart';

/// Mutual exclusion.
/// Acquire and release lock
class Mutex {
  Completer<void>? _completer;

  Future<T> lock<T>(Future<T> Function() function) async {
    await acquire();
    try {
      return await function();
    } finally {
      release();
    }
  }

  Future<void> acquire() {
    final c = _completer;
    if (c == null) {
      _completer = Completer();
      return SynchronousFuture(null);
    } else {
      return c.future;
    }
  }

  void release() {
    final c = _completer;
    if (c == null) throw StateError('cannot release, nothing acquired');
    c.complete();
    _completer = null;
  }
}

/// Run async function successfully exactly once
class RunOnce extends Mutex {
  bool _run = true;

  Future<void> call(Future Function() function) async {
    await acquire();
    try {
      if (_run) {
        await function();
        _run = false;
      }
    } finally {
      release();
    }
  }
}
